// --- tempreture sensor parameters ---
#define HDC1000_ADDRESS 0x40
#define HDC1000_TEMPERATURE_POINTER     0x00
#define HDC1000_HUMIDITY_POINTER        0x01
#define HDC1000_CONFIGURATION_POINTER   0x02
#define HDC1000_SERIAL_ID1_POINTER      0xfb
#define HDC1000_SERIAL_ID2_POINTER      0xfc
#define HDC1000_SERIAL_ID3_POINTER      0xfd
#define HDC1000_MANUFACTURER_ID_POINTER 0xfe

#define HDC1000_CONFIGURE_MSB 0x10 /* Get both temperature and humidity */
#define HDC1000_CONFIGURE_LSB 0x00 /* 14 bit resolution */
// -------------------------------------

// --- full-color led parameters ---
#define DI()      asm("di")
#define EI()      asm("ei")
 
 
#define LED_PINREG  P1.BIT.bit6 // register for rapid access to pin
#define LED_ON      LED_PINREG = 1
#define LED_OFF     LED_PINREG = 0
 
#define LED_ON_WAIT0   NOP();NOP();NOP();NOP();NOP();NOP();
#define LED_OFF_WAIT0  NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();
#define LED_ON_WAIT1   NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();
#define LED_OFF_WAIT1  NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();NOP();


// ---------------------------------------

void connectServer(){
  // Test AT command
  Serial1.println("AT");
  delay(1000);
  if( Serial1.find("OK") ){
    // Connect to WiFi access point.
    //blinkLed(22, 200);
    Serial.print("Connecting to ");
    Serial.println(WLAN_SSID);
    wifi.begin(Serial1);
    if (wifi.joinAP(WLAN_SSID, WLAN_PASS)) {
        //blinkLed(23, 200);
        Serial.print("ok! IP is ");
        String localIp = wifi.getLocalIP();
        Serial.println(localIp);
        
        if (wifi.enableMUX()) {
        Serial.print("multiple ok\r\n");
          if (wifi.createTCP(client, IP, PORT)) {
              Serial.print("create tcp ok\r\n");
          } else {
              Serial.print("create tcp err\r\n");
          }
        
        } else {
            Serial.print("multiple err\r\n");
        }
          
    } else {
        Serial.print("Join AP failure\r\n");
    }
  } else {
    Serial.println("ESP AT command error");  
  }
}

void setUpTemperatureSensor(){
  //Put the HDC1000 IC into the correct operating mode
  Wire.beginTransmission(HDC1000_ADDRESS);
  Wire.write(HDC1000_CONFIGURATION_POINTER);
  Wire.write(HDC1000_CONFIGURE_MSB);
  Wire.write(HDC1000_CONFIGURE_LSB);
  Wire.endTransmission();
}

void getTemperatureAndHumidity(float *temperature, float *humidity) {
  unsigned int tData, hData; // 2byte: 0 ～ 65535

  Wire.beginTransmission(HDC1000_ADDRESS);
  Wire.write(HDC1000_TEMPERATURE_POINTER); 
  Wire.endTransmission();
  while (digitalRead(HDC1000_RDY_PIN) == HIGH);
  Wire.requestFrom(HDC1000_ADDRESS, 4);
  while (Wire.available() < 4);

  tData = Wire.read() << 8;
  tData |= Wire.read();

  hData = Wire.read() << 8;
  hData |= Wire.read();

  *temperature = tData / 65536.0 * 165.0 - 40.0; //ポインタ変数
  *humidity = hData / 65536.0 * 100.0;
}




// ********************************************
void sendLedBuf(uint8_t* red, uint8_t* green, uint8_t* blue, uint8_t num){
  for(int i = 0; i < num; i++){
    sendLedData(*(red + i), *(green + i), *(blue + i));
  }
}
void sendLedData(uint8_t red, uint8_t green, uint8_t blue){
    DI();
    if ( green & 0x80 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( green & 0x40 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( green & 0x20 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( green & 0x10 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( green & 0x08 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( green & 0x04 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( green & 0x02 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( green & 0x01 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
 
    // 襍､LED
    if ( red & 0x80 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( red & 0x40 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( red & 0x20 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( red & 0x10 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( red & 0x08 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( red & 0x04 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( red & 0x02 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( red & 0x01 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
 
    // 髱鱈ED
    if ( blue & 0x80 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( blue & 0x40 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( blue & 0x20 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( blue & 0x10 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( blue & 0x08 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( blue & 0x04 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( blue & 0x02 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
 
    if ( blue & 0x01 )
    {   // on data send
        LED_ON;
        LED_ON_WAIT1;
        LED_OFF;
        LED_OFF_WAIT1;
    }
    else
    {   // off data send
        LED_ON;
        LED_ON_WAIT0;
        LED_OFF;
        LED_OFF_WAIT0;
    }
    EI();
}

