#include <Arduino.h>
#include <doxygen.h>
#include <ESP8266.h>
#include <Wire.h>

// wifi access point
#define WLAN_SSID       "********"
#define WLAN_PASS       "********"
#define IP              "********" // serverのIP
#define PORT            3080 // serverのPORT
#define client 0 // 0 or 1
const int pressureThreshold = 4; 
const int accelThreshold = 20;

// pin numbers
#define LED_RED         22
#define LED_GREEN       23
#define LED_BLUE        24
#define VIB_MOTOR        6
#define HEATER           5
#define PRESSURE_SENSOR A1
#define HDC1000_RDY_PIN 2
#define ACCEL_X_PIN     A3
#define ACCEL_Y_PIN     A4
#define ACCEL_Z_PIN     A5
#define PPG_PIN         A2
#define LED_PIN          3 
#define LED_NUM          3 

ESP8266 wifi;

int cnt = 0; // loop実行回数
float x, y, z; // acceleratiom sensor xyz
float x_old, y_old, z_old;
uint8_t red, blue, green; // eye color
int heartRate = 1000;

// フラグ
boolean myMovingFlag = 0;
boolean myPressedFlag = 0;
boolean yourMovingFlag = 0;
boolean yourPressedFlag = 0;
int stepEye = 0;
boolean isHeart = 0;
boolean isPressedHeart = 0;

// delayを使わず処理を行うために時間データを保持
unsigned long beforeEyeChangeTime;
unsigned long beforeHeartChangeTime;
unsigned long beforePressedHeartChangeTime;
unsigned long yourPressedTime;
unsigned long myPressedTime;


void setup() {
  pinMode( LED_RED, OUTPUT );
  pinMode( LED_GREEN, OUTPUT );
  pinMode( LED_BLUE, OUTPUT );
  pinMode( HEATER, OUTPUT );
  pinMode( VIB_MOTOR, OUTPUT);
  pinMode( LED_PIN, OUTPUT );
  pinMode(HDC1000_RDY_PIN, INPUT);
  
  digitalWrite( LED_RED, HIGH );
  digitalWrite( LED_GREEN, HIGH );
  digitalWrite( LED_BLUE, HIGH );

  Serial.begin(9600);
  Serial1.begin(115200);
  Wire.begin();
  
  connectServer();
  
  setUpTemperatureSensor();
  
  Serial.println("start!");
}


void loop() {
  float x = 0, y = 0, z = 0;
  float temperature, humidity;
  float pressure;
  
  getTemperatureAndHumidity(&temperature, &humidity);
  pressure = analogRead(PRESSURE_SENSOR);
  
  // 加速度は10回分の平均値を算出
  for(int i=0; i<10; i++){
  x += analogRead(ACCEL_X_PIN);
  y += analogRead(ACCEL_Y_PIN);
  z += analogRead(ACCEL_Z_PIN);
  }
  x = x/10;
  y = y/10;
  z = z/10;
  
  // 動いているかどうかを判定
  if(cnt != 0 && ( x-x_old > accelThreshold || x_old-x > accelThreshold || y-y_old > accelThreshold || y_old-y > accelThreshold || z-z_old > accelThreshold || z_old-z > accelThreshold  ) ) {
    myMovingFlag = true;
  } else {
    myMovingFlag = false;
  }
  
  // 加圧されてるかどうかを判定
  if(pressure > pressureThreshold) {
    myPressedFlag = true;
    myPressedTime = millis();
  } else {
    myPressedFlag = false;
  }
  
  // フラグをサーバーに送信＆受信
  sendFlags(myMovingFlag, myPressedFlag);
  receiveFlags(&yourMovingFlag, &yourPressedFlag);
  
  //相手が頻繁にpressしてきたら、心拍数を早めにする。
  if(yourPressedFlag == 1) {
    if( millis() - yourPressedTime < 800 ) {
      heartRate = 700;
    } else {
      heartRate = 1000;
    }
    yourPressedTime = millis();
  }

  // 相手にアクションを起こした（pressした）ときのフィードバックとして、ハートのLEDを白く光らせる。
  if (yourPressedFlag == 0 && yourMovingFlag == 0 && myPressedFlag == 1 && isHeart == 0 && isPressedHeart == 0){
    pressedHeartOn(); 
  }
  if (isPressedHeart == 1 && isHeart == 0 && millis() - beforePressedHeartChangeTime > 500 ) {
    pressedHeartOff();
  }
  
  // 相手が動いていたら、まばたき2回を同じ色で実行
  if(stepEye == 0 && yourMovingFlag == 1 && millis() - beforeEyeChangeTime > 900) {
    colorLot(&red, &blue, &green);
    eyeOn();
  } else if(stepEye == 1 && millis() - beforeEyeChangeTime > 50) {
    eyeOff();
  } else if(stepEye == 2 && millis() - beforeEyeChangeTime > 100) {
    eyeOn();
  } else if(stepEye == 3 && millis() - beforeEyeChangeTime > 50) {
    eyeOff();
  }
  
  // 相手がpressしてきたら、ヒーターをON（42度以下で）
  if(yourPressedFlag == 1 | millis() - yourPressedTime < 6000) {
    Serial.println(temperature);
    if( temperature < 42 ) { 
      digitalWrite(HEATER, HIGH);
    }
    // 心拍動をLEDとバイブで表現
    if(isHeart == 0 && millis() - beforeHeartChangeTime > heartRate) {
      heartOn(); 
    }
  } else {
    digitalWrite(HEATER, LOW);
  }
  
  if(isHeart == 1 && millis() - beforeHeartChangeTime > 100) {
    heartOff();
  } 
  
  cnt++;
  x_old = x;
  y_old = y;
  z_old = z;
  delay(10);
  
}

// まばたき時の目の色をランダムで決める。
void colorLot(uint8_t* red, uint8_t* blue, uint8_t* green) {
  uint8_t buf_LEDs[3];
  for (int j = 0; j < 3; j++) {
    buf_LEDs[j] = random(0, 30);
  }
  *red = buf_LEDs[0];
  *blue = buf_LEDs[1];
  *green = buf_LEDs[2];
}

void eyeOn() {
//  uint8_t buf_LEDs[3];
//  for (int j = 0; j < 3; j++) {
//    buf_LEDs[j] = random(0, 30);
//  }

//  Serial.print(buf_LEDs[0]);
//  Serial.print(" ,");
//  Serial.print(buf_LEDs[1]);
//  Serial.print(" ,");
//  Serial.println(buf_LEDs[2]);a
  
  sendLedData(0, 0, 0);
  for (int i = 1; i < LED_NUM; i++){
    sendLedData(red, blue, green);
  }
  stepEye++;
  if( stepEye > 3) {
    stepEye = 0;
  }
  beforeEyeChangeTime = millis();
}

void eyeOff() {
  for (int i = 0; i < LED_NUM; i++){
    sendLedData(0, 0, 0);
  }
  stepEye++;
  if( stepEye > 3) {
    stepEye = 0;
  }
  beforeEyeChangeTime = millis();
}

void heartOn() {
  sendLedData(120, 30, 150);
  for (int i = 1; i < LED_NUM; i++){
    sendLedData(0, 0, 0);
  }
  if( millis() - myPressedTime < 5000 | myMovingFlag ) {
    digitalWrite(VIB_MOTOR, HIGH);
  }
  isHeart = 1;
  beforeHeartChangeTime = millis();
}

void heartOff() {
  for (int i = 0; i < LED_NUM; i++){
    sendLedData(0, 0, 0);
  }
  digitalWrite(VIB_MOTOR, LOW);
  isHeart = 0;
  beforeHeartChangeTime = millis();
}

void pressedHeartOn() {
  sendLedData(255, 255, 255);
  for (int i = 1; i < LED_NUM; i++){
    sendLedData(0, 0, 0);
  }
  isPressedHeart = 1;
  beforePressedHeartChangeTime = millis();
}

void pressedHeartOff() {
  for (int i = 0; i < LED_NUM; i++){
    sendLedData(0, 0, 0);
  }
  isPressedHeart = 0;
  beforePressedHeartChangeTime = millis();
}

void eyeBlink() {
  sendLedData(0, 0, 0);
  for (int i = 1; i < LED_NUM; i++){
    sendLedData(120, 30, 150);
  }
  delay(300);
  for (int i = 0; i < LED_NUM; i++){
    sendLedData(0, 0, 0);
  }
}


void sendFlags(boolean movingFlag, boolean pressedFlag) {
  String header = String(client) + ",";
  String flags = String(movingFlag) + String(pressedFlag);
  String sendingText = header + flags;
  wifi.send(client, sendingText);
}

void receiveFlags(boolean *movingFlag, boolean *pressedFlag) {
  uint8_t buffer[128] = {0};
  uint32_t len = wifi.recv(buffer, sizeof(buffer), 5000);
  char data[len];
  if (len > 0) {
      for(uint32_t i = 0; i < len; i++) {
        data[i] = (char)buffer[i]; 
      }

//      Serial.print("Received:[");
//      Serial.print(data[0]);
//      Serial.print(data[1]);
//      Serial.print("]\r\n");
      
      if(data[0] == '1'){
        *movingFlag = 1;
      } else {
        *movingFlag = 0;
      }
      
      if(data[1] == '1'){
        *pressedFlag = 1;
      } else {
        *pressedFlag = 0;
      }
  }
}

void debugAccel() {
  Serial.print(x);
  Serial.print(", ");
  Serial.print(y);
  Serial.print(", ");
  Serial.println(z);
}

